@@ -3,21 +3,16 @@ TODO:
     - look into https://github.com/click-contrib/click-help-colors
 """
 
-import datetime
 import json
 import sys
-from typing import Optional
+from typing import List, Optional, Tuple
 import urllib.parse
 import uuid
 
 import click
 import requests
 
-from timer_cli.auth import get_access_token
-
-
-# how long to wait before giving up on requests to the API
-TIMEOUT = 10
+from timer_cli.job import job_delete, job_list, job_status, job_submit
 
 
 def show_usage(cmd: click.Command):
@@ -27,6 +22,7 @@ def show_usage(cmd: click.Command):
     The actual usage message is also specific to incomplete commands.
     """
     ctx = click.get_current_context()
+    ctx.max_content_width = 100
     formatter = ctx.make_formatter()
     cmd.format_help_text(ctx, formatter)
     cmd.format_options(ctx, formatter)
@@ -42,11 +38,6 @@ def show_response(response: requests.Response):
     click.echo(json.dumps(response.json()))
 
 
-def handle_requests_exception(e: Exception):
-    click.echo(f"error in request: {e}", err=True)
-    sys.exit(1)
-
-
 class Command(click.Command):
     """
     Subclass click.Command to show help message if a command is missing arguments.
@@ -124,15 +115,6 @@ class URL(click.ParamType):
         return value
 
 
-def get_headers(token_store: Optional[str] = None) -> dict:
-    """
-    Assemble any needed headers that should go in all requests to the timer API, such
-    as the access token.
-    """
-    access_token = get_access_token(token_store=token_store)
-    return {"Authorization": f"Bearer {access_token}"}
-
-
 cli = click.Group()
 
 
@@ -146,21 +128,27 @@ def job():
     "--name",
     required=True,
     type=str,
-    help="name to identify this job (not necessarily unique)",
+    help="Name to identify this job (not necessarily unique)",
 )
 @click.option(
     "--start",
     required=False,
     type=click.DateTime(),
     help=(
-        "start time for the job (defaults to current time)"
+        "Start time for the job (defaults to current time)"
     ),
 )
 @click.option(
     "--interval",
     required=True,
     type=int,
-    help="interval in seconds at which the job should run",
+    help="Interval in seconds at which the job should run",
+)
+@click.option(
+    "--scope",
+    required=True,
+    type=str,
+    help="Globus Auth scope needed for this action",
 )
 @click.option(
     "--action-url",
@@ -197,46 +185,23 @@ def submit(
     name: str,
     start: Optional[click.DateTime],
     interval: int,
+    scope: str,
     action_url: urllib.parse.ParseResult,
     action_body: Optional[str],
     action_file: Optional[click.File],
 ):
     """
-    Submit a new job.
+    Submit a job.
     """
-    callback_body = None
-    try:
-        if action_body:
-            callback_body = action_body.strip("'").strip('"')
-            callback_body = json.loads(action_body)
-        else:  # action_file
-            callback_body = json.load(action_file)
-    except (TypeError, ValueError) as e:
-        raise click.BadOptionUsage(
-            "action-body",
-            f"--action-body must parse into valid JSON; got error: {e}",
-        )
-    start = start or datetime.datetime.now()
-    callback_url: str = action_url.geturl()
-    req_json = {
-        "name": name,
-        "start": start.isoformat(),
-        "interval": interval,
-        "callback_url": callback_url,
-        "callback_body": callback_body,
-    }
-    headers = get_headers()
-    try:
-        response = requests.post(
-            "https://sandbox.timer.automate.globus.org/jobs/",
-            json=req_json,
-            headers=headers,
-            timeout=TIMEOUT,
-        )
-    except requests.RequestException as e:
-        handle_requests_exception(e)
-        return
-    show_response(response)
+    show_response(job_submit(
+        name,
+        start,
+        interval,
+        scope,
+        action_url,
+        action_body,
+        action_file,
+    ))
 
 
 @job.command(cls=Command)
@@ -244,17 +209,7 @@ def list():
     """
     List submitted jobs.
     """
-    headers = get_headers()
-    try:
-        response = requests.get(
-            f"https://sandbox.timer.automate.globus.org/jobs/",
-            headers=headers,
-            timeout=TIMEOUT,
-        )
-    except requests.RequestException as e:
-        handle_requests_exception(e)
-        return
-    show_response(response)
+    show_response(job_list())
 
 
 @job.command(cls=Command)
@@ -263,33 +218,124 @@ def status(job_id: uuid.UUID):
     """
     Return the status of the job with the given ID.
     """
-    headers = get_headers()
-    try:
-        response = requests.get(
-            f"https://sandbox.timer.automate.globus.org/jobs/{job_id}",
-            headers=headers,
-            timeout=TIMEOUT,
-        )
-    except requests.RequestException as e:
-        handle_requests_exception(e)
-        return
-    show_response(response)
+    show_response(job_status(job_id))
 
 
 @job.command(cls=Command)
 @click.argument("job_id", type=uuid.UUID)
 def delete(job_id: uuid.UUID):
-    headers = get_headers()
-    try:
-        response = requests.delete(
-            f"https://sandbox.timer.automate.globus.org/jobs/{job_id}",
-            headers=headers,
-            timeout=TIMEOUT,
-        )
-    except requests.RequestException as e:
-        handle_requests_exception(e)
-        return
-    show_response(response)
+    show_response(job_delete(job_id))
+
+
+@job.command(cls=Command)
+@click.option(
+    "--name",
+    required=True,
+    type=str,
+    help="Name to identify this job (not necessarily unique)",
+)
+@click.option(
+    "--start",
+    required=False,
+    type=click.DateTime(),
+    help=(
+        "Start time for the job (defaults to current time)"
+    ),
+)
+@click.option(
+    "--interval",
+    required=True,
+    type=int,
+    help="Interval in seconds at which the job should run",
+)
+@click.option(
+    "--source-endpoint",
+    required=True,
+    type=str,
+    help="ID for the source transfer endpoint",
+)
+@click.option(
+    "--dest-endpoint",
+    required=True,
+    type=str,
+    help="ID for the destination transfer endpoint",
+)
+@click.option(
+    "--label",
+    required=False,
+    type=str,
+    help=(
+        "An optional label for the transfer operation, up to 128 characters long. Must"
+        " contain only letters/numbers/spaces, and the following characters: - _ ,"
+    ),
+)
+@click.option(
+    "--sync-level",
+    required=False,
+    type=int,
+    help=(
+        "Specify that only new or modified files should be transferred. The behavior"
+        " depends on the value of this parameter, which must be a value 0--3, as"
+        " defined in the transfer API: 0. Copy files that do not exist at the"
+        " destination. 1.  Copy files if the size of the destination does not match the"
+        " size of the source. 2. Copy files if the timestamp of the destination is"
+        " older than the timestamp of the source. 3. Copy files if checksums of the"
+        " source and destination do not match."
+    ),
+)
+@click.option(
+    "--item",
+    required=True,
+    type=(str, str, bool),
+    multiple=True,
+    help=(
+        "Used to specify the transfer items; provide as many of this option as files"
+        " to transfer. The format for this option is `--item SRC DST RECURSIVE`, where"
+        " RECURSIVE specifies, if this item is a directory, to transfer the entire"
+        " directory. For example: `--item ~/file1.txt ~/new_file1.txt false`"
+    ),
+)
+def transfer(
+    name: str,
+    start: Optional[click.DateTime],
+    interval: int,
+    source_endpoint: str,
+    dest_endpoint: str,
+    label: Optional[str],
+    sync_level: Optional[int],
+    item: List[Tuple[str, str, Optional[str]]],
+):
+    """
+    Submit specifically a transfer job. The options for this command are tailored to
+    the transfer action.
+    """
+    action_url = urllib.parse.urlparse(
+        "https://actions.automate.globus.org/transfer/transfer/run"
+    )
+    scope = "https://auth.globus.org/scopes/actions.globus.org/transfer/transfer"
+    transfer_items = [
+        {"source_path": i[0], "destination_path": i[1], "recursive": i[2]}
+        for i in item
+    ]
+    action_body = {
+        "source_endpoint_id": source_endpoint,
+        "destination_endpoint_id": dest_endpoint,
+        "transfer_items": transfer_items,
+    }
+    if label:
+        action_body["label"] = label
+    if sync_level:
+        action_body["sync_level"] = sync_level
+    show_response(job_submit(
+        name,
+        start,
+        interval,
+        scope,
+        action_url,
+        action_body=None,
+        action_file=None,
+        callback_body=action_body,
+    ))
 
 
 def main():
